/** EnumPlugIns.java enumerates
 * plugins at program start-up, and
 * contains functions for managing
 * them once the program is loaded
 * $Id: EnumPlugIns.java 1.14 2003/03/02 06:14:22 jeffnik Exp $
*/

/* JamochaMUD, a Muck/Mud client program
 * Copyright (C) 1998-2003  Jeff Robinson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package anecho.JamochaMUD;

import java.awt.List;
import java.awt.event.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.ObjectInputStream;

import java.util.Properties;
import java.util.Vector;

import anecho.gui.PosTools;

import anecho.JamochaMUD.plugins.PlugInterface;

	/** EnumPlugIns.java enumerates
	 * plugins at program start-up, and
	 * contains functions for managing
	 * them once the program is loaded
         * @version $Id: EnumPlugIns.java 1.14 2003/03/02 06:14:22 jeffnik Exp $
         * @author Jeff Robinson
 	 */
public class EnumPlugIns{

	private String pathSeparator;
	static private String temp1, temp2;
        public static Vector plugInClass = new Vector(0,1);
        public static Vector plugInName = new Vector(0,1);
        public static Vector plugInType = new Vector(0,1);
        public static Vector plugInStatus = new Vector(0,1);
        private static Object plugClass;
        private static String userDirectory;
        private static Vector temp = new Vector(0,1);
        private JMConfig settings;

	/**
	 * This is the startup class to
	 * enumerate all the plugins found
	 * in the plug-in directory
	 */
        public EnumPlugIns(JMConfig mainSettings) {
            this.settings = mainSettings;
            // LoadPlugIns();
        }

        public void LoadPlugIns() {
            plugInClass.removeAllElements();
            plugInName.removeAllElements();
            plugInType.removeAllElements();
            plugInStatus.removeAllElements();
            temp.removeAllElements();
            String temp1 = new String("");
            String temp2 = new String("");

            pathSeparator = settings.getPathSeparator();

            // Now set the plug-ins directory
            userDirectory = settings.getUserDirectory();
            File plugIns = new File(userDirectory + "plugins");

            // check and see if there is a 'plugins' directory
            if (!plugIns.exists()) {
                // There is no plugins directory, so we'll create one
                plugIns.mkdir();
            }

            // Now, let's reset the menu on the main window
            MuckMain tempMain = settings.getMainWindowVariable();
            tempMain.removeAllPlugins();

            // Get a list of the files in that directory
            // discarding those that aren't plug-ins.

            String fileList[];
            // fileList = plugIns.list();

            // If we are set for a multi-user environment, we'll also need to
            // read the list of plug-ins from the "master" plug-ins directory
            if (!settings.isSingleUserMode()) {
                String workingDirectory = settings.getWorkingDirectory();

                // This is really really fugly!  By throwing an intentional error here
                // we can get the real path that we were looking for!
                try {
                    Class testC = this.getClass();
                    java.io.InputStream tempStream = testC.getResourceAsStream("plugins");

                    // If the first command doesn't throw an exception then we can try the following:
                    java.net.URL tempURL = testC.getResource("plugins");
                    workingDirectory = tempURL.getFile();
                    System.out.println("UM: Master plug-in directory: " + workingDirectory);
                } catch (Exception e) {
                    // System.out.println("AsStream exception: " + e);
                    workingDirectory = e.getMessage();
                    workingDirectory.trim();
                    System.out.println("EM: Master plug-in directory: " + workingDirectory);
                }

                // File mainPlugs = new File(workingDirectory + "plugins");
                // System.out.println("tempURL.getFile: " + tempURL.getFile());
                // System.out.println("tempStream: " + tempStream);
                // mainPlugs = new File(tempURL.getFile());
                File mainPlugs = new File(workingDirectory);

                String tempList[], oldList[];
                tempList = mainPlugs.list();
                oldList = plugIns.list();

                // Copy the fileList into the oldList;
                int first = oldList.length;
                int second;
                if (tempList != null) {
                    second = tempList.length;
                } else {
                    second = 0;
                }

                fileList = new String[first + second];


                System.arraycopy(oldList, 0, fileList, 0, oldList.length);
                if (second > 0) {
                    System.arraycopy(tempList, 0, fileList, oldList.length, tempList.length);
                }

            } else {
                fileList = plugIns.list();
            }

            // Now read this to an array, deleting any
            // non-'class' files.  This could be done
            // with a FilenameFilter, but we're going
            // to do this quick and dirty.

            try {
                // This may cause an error if the file doesn't exist
                // Specifically, this occurs on *nix-type systems

                for(int i = 0; i < fileList.length; i++) {
                    if (fileList[i].toLowerCase().endsWith("class")) {
                        fileList[i] = fileList[i].substring(0, (fileList[i].length()) - 6);
                        // This one has made the temp list!
                        // Now see if it conforms to the
                        // Plug-In 'standard'
                        // This must be done through 'type-casting',
                        // for the plug-ins don't exist at compile time
                        try {
                            // First, capture the class
                            plugClass = (Object)Class.forName("anecho.JamochaMUD.plugins." + fileList[i]).newInstance();
                            PlugInterface plug = (PlugInterface)plugClass;

                            // Querry to see if it's a plugin or not
                            try {
                                plugInClass.addElement(plugClass);
                                plugInName.addElement(plug.PlugInName());

                                tempMain.addPlugin(plug.PlugInName(), plug.HasProperties());

                                plugInType.addElement(plug.PlugInType());
                                plugInStatus.addElement("false");

                                // tell the plug-in the variable for our 'settings'
                                plug.setSettings(settings);

                                // New as of 2002, create a settings directory automatically
                                // This should help promote good form and consistency.
                                if (plug.haveConfig()) {
                                    createSettingsDirectory(fileList[i]);
                                }

                                plug.InitialiseAtLoad();
                            } catch (Exception notPlugIn) {
                                // Disregarded
                                System.out.println("Name failure... " + notPlugIn + " from " + plugClass);
                                notPlugIn.printStackTrace();
                            }
                        } catch (InstantiationException ie) {
                            // This is a standard error, from trying
                            // to instantiate the 'interface'
                        } catch (Exception e) {
                            System.out.println("Plugin exception: " + e);
                        }

                    }
                }

            } catch (Exception err) {
                // This is from the 'outside' loop
                System.out.println("EnumPlugins: Non-fatal error " + err);
            }

            // Write the names and status to the hashtable
            settings.setPlugInName(plugInName);
            settings.setPlugInStatus(plugInStatus);

            ResetPlugInStatus();
        }

        /**
         * Analyse the input/output and cycle through the list of plugins,
	 * applying any where the input/output meets the right criteria
	 */
        public static String CallPlugin(String targetString, String plugType, MuSocket mu) {

            if (targetString == null) {
                return null;
            }

            // This is a public routine called to parse the plugins
            for (int i = 0; i < plugInName.size(); i++) {
			// Cycle through the list of plugins
			// calling only the type appropriate (input/output/other)
			temp1 = (String)plugInType.elementAt(i);
			temp2 = (String)plugInStatus.elementAt(i);			

			if (plugType.toLowerCase().equals(temp1.toLowerCase()) && temp2.toLowerCase().equals("true")) {
                            // This is what we want, call the plugin
				
                            try {
                                plugClass = plugInClass.elementAt(i);
                            } catch (Exception e) {
                                System.out.println("Plugin Exception " + e);
                            }
                            targetString = new String(((PlugInterface)plugClass).PlugMain(targetString, mu));
                        }
            }

	return targetString;
	}

	/** 
	 * Set the status of the plugins, whether they are active or inactive
	 */
        // public static void ResetPlugInStatus() {
        public void ResetPlugInStatus() {
		// Read in the .plugins.rc file, to set the state of the plugins
		// Vector stateVector;
		// List stateList;		
		Vector stateList;

		try {
                    FileInputStream inputFile = new FileInputStream(userDirectory + ".plugins.rc");
                    ObjectInputStream serializeStream = new ObjectInputStream(inputFile);
                    // stateList = (List) serializeStream.readObject();
                    stateList = (Vector) serializeStream.readObject();
                } catch (FileNotFoundException e) {
                    // The file does not exist, but will be written later
                    // This is not a bad thing.  We'll just keep the error quiet as
                    // Otherwise folks found it disturbing
                    // System.out.println(".plugins.rc file was not found. Don't worry, we'll create one later on.");
                    return;
                } catch (Exception e) {
                    System.out.println("Error during serialization " + e);
                    // The file probably does not exist
                    return;
                }

		// Now that we have the vector, loop through and change
		// any needed plugins to 'true' (active)
		for (int j = 0; j < plugInName.size(); j++) {
			for (int i = 0; i < stateList.size(); i++) {
				if (plugInName.elementAt(j).equals(stateList.elementAt(i))) {
					// This matches, so we set it true
					plugInStatus.setElementAt("true", j);
				}
			}
		}
		
		// Now add the plugInStatus to the hashtable
                settings.setPlugInName(plugInName);
                settings.setPlugInStatus(plugInStatus);
	}
	/**
	 * Analyse the input/output and cycle through the list of plugins,
	 * applying any where the input/output meets the right criteria
	 */
	public static String Description(String targetName) {

		String targetString = "";

		// This is a public routine called to parse the plugins
		for (int i = 0; i < plugInName.size(); i++) {
			// Cycle through the list of plugins
			// calling only the type appropriate (input/output/other)
			temp1 = (String)plugInName.elementAt(i);

			if (targetName.toLowerCase().equals(temp1.toLowerCase())) {
				// This is what we want, call the plugin
				
				try {
					plugClass = plugInClass.elementAt(i);
				} catch (Exception e) {
					System.out.println("Plugin Description Exception " + e);
				}
				targetString = new String(((PlugInterface)plugClass).PlugInDescription());
			}
		}
	return targetString;
	}

        /** Return the plugin object for the name given */
        public static Object classByName(String targetName) {
            // This is a public routine called to parse the plugins
            for (int i = 0; i < plugInName.size(); i++) {
                // Cycle through the list of plugins
                // calling only the type appropriate (input/output/other)
                temp1 = (String)plugInName.elementAt(i);

                if (targetName.toLowerCase().equals(temp1.toLowerCase())) {
                    // This is what we want, call the plugin

                    try {
                        plugClass = plugInClass.elementAt(i);
                    } catch (Exception e) {
                        System.out.println("Plugin Description Exception " + e);
                    }
                    // targetString = new String(((PlugInterface)plugClass).PlugInDescription());
                }
            }

            return plugClass;
        }

        /** This is a static method that we'll
         * allow our plugins to call via
         * the "loader" in our PlugInterface.
         * This basically allows our plug-ins to send
         * responses back to the connected MUD/Muck */
        public static void SendToMU(String textToMU) {
            // Since we have this function nicely mapped out already, we'll
            // just call it from here
            // DataIn.JMSendText(textToMU);
            // Fix this XXX
            // MuckConn.typeHere.JMSendText(textToMU);
        }

        /** A static method to allow a plug-in to
         * write output to the screen.  There may
         * need to be more done than what can simply
         * be done at the time of &quot;processing"
         */
        public static void Write(String text) {
            // Fix this XXX
            // MuckMain.mainText.append(text);
        }

        public JMConfig getSettings() {
            return settings;
        }

        /** Check to see if this plug-in has a settings directory.
         *  If one isn't found the we'll create it
         */
        private void createSettingsDirectory(String name) {
            // check to see if the directory already exists
            String fileName = new String(userDirectory + "plugins" + pathSeparator + name + "Dir");
            File settingsDir = new File(fileName);

            // check and see if there is a 'plugins' directory
            if (!settingsDir.exists()) {
                // There is no plugins directory, so we'll create one
                settingsDir.mkdir();
            }


        }
}			


