/**
 * JMConfig.java, an object to keep and share
 * JamochaMUD settings between classes
 * $Id: JMConfig.java 1.18 2003/03/01 21:41:53 jeffnik Exp $
 */

/* JamochaMUD, a Muck/Mud client program
 * Copyright (C) 1998-2001  Jeff Robinson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * vesion 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package anecho.JamochaMUD;

import anecho.JamochaMUD.DataIn;
import anecho.JamochaMUD.CHandler;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.Rectangle;

import java.util.Hashtable;
import java.util.Vector;

public class JMConfig {

    // This hashtable will contain all the values we should need
    private Hashtable vars;
    private Hashtable userVariables;    // variables defined by the user
    private Hashtable userDefs; // Definitions defined by the user

    public static final String AUTOFOCUSINPUT = "AutoFocusInput";
    public static final String ALTFOCUS = "AltFocus";
    public static final String BACKGROUNDCOLOUR = "BackgroundColour";
    public static final String BROWSER1 = "Browser1";
    public static final String BROWSER2 = "Browser2";
    public static final String CONNECTIONHANDLER = "ConnectionHandler";
    public static final String CONNMUCK = "ConnMuck";
    public static final String CONNPORT = "ConnPort";
    public static final String DATABAR = "DataBar";
    public static final String DATAINVARIABLE = "DataInVariable";
    public static final String DOUBLEBUFFER = "DoubleBuffer";
    public static final String EMAILCLIENT = "EMailClient";
    public static final String ENUMERATOR = "Enumerator";
    public static final String FONTFACE = "FontFace";
    public static final String FOREGROUNDCOLOUR = "ForegroundColour";
    public static final String FTPCLIENT = "FTPClient";
    public static final String ICONIMAGE = "IconImage";
    public static final String JMUDCORE = "JamochaMUDCore";
    public static final String LASTMU = "LastConnectedMU";
    public static final String LOCALECHO = "LocalEcho";
    public static final String MACROFRAME = "MacroFrame";
    public static final String MACRODEFS = "MacroDefs";
    public static final String MACROLABELS = "MacroLabels";
    public static final String MACROVISIBLE = "MacroVisible";
    public static final String MAINWINDOW = "MainWindow";
    public static final String MAINWINDOWICONIFIED = "MainWindowIconified";
    public static final String MAINWINDOWVARIABLE = "MainWindowVariable";
    public static final String MUCKMAINTITLE = "MuckMainTitle";
    public static final String MUCKNAME = "MuckName";
    public static final String MUCKADDY = "MuckAddy";
    public static final String MUCKPORT = "MuckPort";
    public static final String OSNAME = "OSName";
    public static final String PATHSEPARATOR = "PathSeparator";
    public static final String PLUGENUMERATOR = "PluginEnumerator";
    public static final String PLUGINNAME = "PlugInName";
    public static final String PLUGINSTATUS = "PlugInStatus";
    public static final String PROXY = "ProxyEnabled";
    public static final String PROXYHOST = "ProxyHost";
    public static final String PROXYPORT = "ProxyPort";
    public static final String RELEASEPAUSE = "ReleasePause";
    public static final String SCREENSIZE = "ScreenSize";
    public static final String SINGLEUSERMODE = "Single User Mode";
    public static final String SPLITVIEW = "SplitView";
    public static final String SYNCWINDOWS = "SyncWindows";
    public static final String TFKEYEMU = "TFKeyEmu";
    public static final String TIMERS = "Timers";
    public static final String TIMERSVISIBLE = "TimersVisible";
    public static final String UNICODE = "UseUnicode";
    public static final String USERDIRECTORY = "UserDirectory";
    public static final String USERPLUGINDIR = "UserPlugInDir";
    public static final String WORKINGDIRECTORY = "WorkingDirectory";
    public static final String BUNDLEBASE = "anecho.JamochaMUD.JamochaMUDBundle";

    /** Our constructor is empty, and we like it that way!
     * Well, actually, all our variables should have some base value
     * straight &quot;out of the box".  There would be far too many
     * to do in the constructor... without falling asleep.
     *
     * Individual "get" statements will return a default value if
     * one does not exist.  This should avoid problems with a damaged
     * .jamocha.rc, missing information when writing out .jamocha.rc
     * or like scenarios.
     */
    public JMConfig() {
        vars = new Hashtable(101, .75f);
        userVariables = new Hashtable(5, .75f);
        userDefs = new Hashtable(5, .75f);
    }

    /** Check our hashtable for the existance
     * of key &quot;String"
     */
    private boolean checkKey(String key) {
        if (vars.containsKey(key)) {
            return true;
        }
        return false;
    }

    /** Convert a string value to a boolean */
    private boolean StringToboolean(String key) {
        String workString = (String)vars.get(key);
        if (workString.toLowerCase().equals("true")) {
            return true;
        }
        return false;
    }

    /* Convert a boolean value to a string */
    private String booleanToString(boolean status) {
        if (status) {
            return "true";
        }

        return "false";
    }


    public void setJMboolean(String item, String setting) {
        String value = "false";
        if (setting.toLowerCase().equals("true")) {
            // setAutoFocusInput(true);
            value = "true";
        }
        vars.put(item, value);
    }

    public synchronized void setJMObject(String item, Object value) {
        vars.put(item, value);
    }

    public synchronized Object getJMObject(String item) {
        if (checkKey(item)) {
            return (Object)vars.get(item);
        }
        return null;
    }

    public synchronized String getJMString(String item) {
        if (checkKey(item)) {
            return (String)vars.get(item);
        }
        return null;
    }

    public synchronized boolean getJMboolean(String item) {
        if (checkKey(item)) {
            return StringToboolean(item);
        }
        return false;
    }

    /* All our happy little methods */
    public synchronized void setAutoFocusInput(boolean setting) {
        setAutoFocusInput(setting + "");
    }

    public synchronized void setAutoFocusInput(String setting) {
        setJMboolean(AUTOFOCUSINPUT, setting);
    }

    public synchronized boolean getAutoFocusInput() {
        return getJMboolean(AUTOFOCUSINPUT);
    }

    public synchronized void setForegroundColour(Color fgColor) {
        vars.put(FOREGROUNDCOLOUR, fgColor);
    }

    public synchronized Color getForegroundColour() {
        if (checkKey(FOREGROUNDCOLOUR)) {
            return (Color)getJMObject(FOREGROUNDCOLOUR);
        } else {
            return new Color(0, 255, 85);
        }
    }

    public synchronized void setBackgroundColour(Color bgColor) {
        vars.put(BACKGROUNDCOLOUR, bgColor);
    }

    public synchronized Color getBackgroundColour() {
        if (checkKey(BACKGROUNDCOLOUR)) {
            return (Color)getJMObject(BACKGROUNDCOLOUR);
        } else {
            return new Color(0, 0, 0);
        }
    }

    public synchronized void setFontFace(Font fFace) {
        vars.put(FONTFACE, fFace);
    }

    public synchronized Font getFontFace() {
        if (checkKey(FONTFACE)) {
            return (Font)getJMObject(FONTFACE);
        } else {
            return new Font("Monospaced", 0, 12);
        }
    }

    /** Set the stat for our browser, as indicated by
     * <TT>String browser</TT>.  The path to the
     * executable will be <TT>String program</TT>
     */
    public synchronized void setBrowser(String browser, String program) {
        vars.put(browser, program);
    }

    /** Return the path of the executable for the
     * requested browser.
     */
    public synchronized String getBrowser(String browser) {
        return getJMString(browser);
    }

    public synchronized void setBrowserInstance(String browser, String status) {
        vars.put(browser + "Instance", status);
    }

    public synchronized void setBrowserInstance(String browser, boolean status) {
//        if (status) {
//            setBrowserInstance(browser, "true");
//        } else {
//            setBrowserInstance(browser, "false");
        //        }
        setBrowserInstance(browser, booleanToString(status));
    }

    public synchronized boolean getBrowserInstance(String browser) {
        return getJMboolean(browser + "Instance");
    }

    public synchronized void setSyncWindows(String status) {
        vars.put(SYNCWINDOWS, status);
    }

    public synchronized void setSyncWindows(boolean status) {
//        if (status) {
//            setSyncWindows("true");
//        } else {
//            setSyncWindows("false");
        //        }
        setSyncWindows(booleanToString(status));
    }

    public synchronized boolean getSyncWindows() {
        return getJMboolean(SYNCWINDOWS);
    }

    public synchronized void setTFKeyEmu(boolean status) {
//        if (status) {
//            setTFKeyEmu("true");
//        } else {
//            setTFKeyEmu("false");
        //        }
        setTFKeyEmu(booleanToString(status));
    }

    public synchronized void setTFKeyEmu(String status) {
        vars.put(TFKEYEMU, status);
    }

    public synchronized boolean getTFKeyEmu() {
        return getJMboolean(TFKEYEMU);
    }

    public synchronized void setDoubleBuffer(String status) {
        vars.put(DOUBLEBUFFER, status);
    }

    public synchronized void setDoubleBuffer(boolean status) {
        setDoubleBuffer(booleanToString(status));
    }

    public synchronized boolean getDoubleBuffer() {
        return getJMboolean(DOUBLEBUFFER);
    }

    public synchronized void setUseUnicode(String status) {
        vars.put(UNICODE, status);
    }

    public synchronized void setUseUnicode(boolean status) {
        setUseUnicode(booleanToString(status));
    }

    public synchronized boolean getUseUnicode() {
        return getJMboolean(UNICODE);
    }

    public synchronized void setMacroVisible(String status) {
        vars.put(MACROVISIBLE, status);
    }

    public synchronized void setMacroVisible(boolean status) {
        setMacroVisible(booleanToString(status));
    }

    public synchronized boolean getMacroVisible() {
        return getJMboolean(MACROVISIBLE);
    }

    public synchronized void setMacroFrame(Rectangle mf) {
        vars.put(MACROFRAME, mf);
    }

    public synchronized Rectangle getMacroFrame() {
        // return (Rectangle)getJMObject(MACROFRAME);
        if (checkKey(MACROFRAME)) {
            return (Rectangle)getJMObject(MACROFRAME);
        } else {
            return new Rectangle(445, 30, 175, 165);
        }
    }

    public synchronized void setTimersVisible(String status) {
        vars.put(TIMERSVISIBLE, status);
    }

    public synchronized void setTimersVisible(boolean status) {
        setTimersVisible(booleanToString(status));
    }

    public synchronized boolean getTimersVisible() {
        return getJMboolean(TIMERSVISIBLE);
    }

    public synchronized void setMacroLabels(String[] labels) {
        vars.put(MACROLABELS, labels);
    }

    public synchronized String[] getMacroLabels() {
        return (String[])vars.get(MACROLABELS);
    }

    public synchronized void setMacroDefs(String[] defs) {
        vars.put(MACRODEFS, defs);
    }

    public synchronized String[] getMacroDefs() {
        return (String[])vars.get(MACRODEFS);
    }

    public synchronized void setUserDirectory(String dir) {
        vars.put(USERDIRECTORY, dir);
    }

    public synchronized String getUserDirectory() {
        return getJMString(USERDIRECTORY);
    }

    public synchronized void setUserPlugInDir(String dir) {
        vars.put(USERPLUGINDIR, dir);
    }

    public synchronized String getUserPlugInDir() {
        return getJMString(USERPLUGINDIR);
    }

    public synchronized void setPathSeparator(String sep) {
        vars.put(PATHSEPARATOR, sep);
    }

    public synchronized String getPathSeparator() {
        return getJMString(PATHSEPARATOR);
    }

    public synchronized void setWorkingDirectory(String dir) {
        vars.put(WORKINGDIRECTORY, dir);
    }

    public synchronized String getWorkingDirectory() {
        return getJMString(WORKINGDIRECTORY);
    }

    public synchronized void setConnMuck(String muck) {
        vars.put(CONNMUCK, muck);
    }

    public synchronized String getConnMuck() {
        return getJMString(CONNMUCK);
    }

    public synchronized void setMainWindow(Rectangle main) {
        vars.put(MAINWINDOW, main);
    }

    public synchronized String getLastMU() {
        return getJMString(LASTMU);
    }

    public synchronized void setLastMU(String last) {
        vars.put(LASTMU, last);
    }

    public synchronized Rectangle getMainWindow() {
        if (checkKey(MAINWINDOW)) {
            return (Rectangle)getJMObject(MAINWINDOW);
        } else {
            return new Rectangle(0, 10, 600, 355);
        }
    }

    public synchronized void setDataBar(Rectangle bar) {
        vars.put(DATABAR, bar);
    }

    public synchronized Rectangle getDataBar() {
        if (checkKey(DATABAR)) {
            return (Rectangle)getJMObject(DATABAR);
        } else {
            return new Rectangle(0, 385, 600, 60);
        }
    }

    public synchronized void setTimers(Rectangle bar) {
        vars.put(TIMERS, bar);
    }

    public synchronized Rectangle getTimers() {
        if (checkKey(TIMERS)) {
            return (Rectangle)getJMObject(TIMERS);
        } else {
            return new Rectangle(500, 300, 120, 100);
        }
    }

    public synchronized void setIconImage(Image icon) {
        vars.put(ICONIMAGE, icon);
    }

    public synchronized Image getIconImage() {
        return (Image)getJMObject(ICONIMAGE);
    }

    public synchronized void setPlugInName(Vector names) {
        vars.put(PLUGINNAME, names);
    }

    public synchronized Vector getPlugInName() {
        return (Vector)getJMObject(PLUGINNAME);
    }

    public synchronized void setPlugInStatus(Vector status) {
        vars.put(PLUGINSTATUS, status);
    }

    public synchronized Vector getPlugInStatus() {
        return (Vector)getJMObject(PLUGINSTATUS);
    }

    public synchronized void setConnPort(int port) {
        vars.put(CONNPORT, port + "");
    }

    public synchronized int getConnPort() {
        if (checkKey(CONNPORT)) {
            return Integer.parseInt((String)vars.get(CONNPORT));
        }
        return 0;
    }

    public synchronized void setMuckName(Vector name) {
        vars.put(MUCKNAME, name);
    }

    public synchronized Vector getMuckName() {
        if (!checkKey(MUCKNAME)) {
            // Create a default MU* setting
            createNewMuck();
        }
        return (Vector)getJMObject(MUCKNAME);
    }

    public synchronized void setMuckAddy(Vector addy) {
        vars.put(MUCKADDY, addy);
    }

    public synchronized Vector getMuckAddy() {
        if (!checkKey(MUCKADDY)) {
            // Create a default MU* setting
            createNewMuck();
        }
        return (Vector)getJMObject(MUCKADDY);
    }

    public synchronized void setMuckPort(Vector port) {
        vars.put(MUCKPORT, port);
    }

    public synchronized Vector getMuckPort() {
        if (!checkKey(MUCKPORT)) {
            // Create a default MU* setting
            createNewMuck();
        }
        return (Vector)getJMObject(MUCKPORT);
    }

    public synchronized void setScreenSize(Dimension screen) {
        vars.put(SCREENSIZE, screen);
    }

    public synchronized Dimension getScreenSize() {
        return (Dimension)getJMObject(SCREENSIZE);
    }

    public synchronized void setConnectionHandler(CHandler connection) {
        vars.put(CONNECTIONHANDLER, connection);
    }

    public synchronized CHandler getConnectionHandler() {
        return (CHandler)getJMObject(CONNECTIONHANDLER);
    }

    public synchronized void setProxy(boolean status) {
        setProxy(booleanToString(status));
    }

    public synchronized void setProxy(String status) {
        vars.put(PROXY, status);
    }

    public synchronized boolean getProxy() {
        return getJMboolean(PROXY);
    }

    public synchronized void setProxyHost(String host) {
        vars.put(PROXYHOST, host);
    }

    public synchronized String getProxyHost() {
        if (checkKey(PROXYHOST)) {
            return (String)vars.get(PROXYHOST);
        }
        return "";
    }

    public synchronized void setProxyPort(int port) {
        setProxyPort(port + "");
    }

    public synchronized void setProxyPort(String port) {
        vars.put(PROXYPORT, port);
    }

    public synchronized int getProxyPort() {
        if (checkKey(PROXYPORT)) {
            try {
                int port = Integer.parseInt((String)vars.get(PROXYPORT));
                return port;
            } catch(Exception e) {
                // return Integer.parseInt((String)vars.get(PROXYPORT));
                System.out.println("Our proxyport is: " + (String)vars.get(PROXYPORT));
            }
        }
        return 0;
    }

    public synchronized void setEnumerator(EnumPlugIns enum) {
        vars.put(ENUMERATOR, enum);
    }

    public synchronized EnumPlugIns getEnumerator() {
        return (EnumPlugIns)getJMObject(ENUMERATOR);
    }

    public synchronized void setJMCore(JMUD core) {
        vars.put(JMUDCORE, core);
    }

    public synchronized JMUD getJMCore() {
        return (JMUD)getJMObject(JMUDCORE);
    }

    public synchronized void setDataInVariable(DataIn variable) {
        vars.put(DATAINVARIABLE, variable);
    }

    public synchronized DataIn getDataInVariable() {
        return (DataIn)getJMObject(DATAINVARIABLE);
    }

    public synchronized void setMainWindowVariable(MuckMain variable) {
        vars.put(MAINWINDOWVARIABLE, variable);
    }

    public synchronized MuckMain getMainWindowVariable() {
        return (MuckMain)getJMObject(MAINWINDOWVARIABLE);
    }

    public synchronized void setMuckMainTitle(String title) {
        vars.put(MUCKMAINTITLE, title);
    }

    public synchronized String getMuckMainTitle() {
        if (checkKey(MUCKMAINTITLE)) {
            return (String)vars.get(MUCKMAINTITLE);
        }
        return "";
    }

    public synchronized void setPlugEnumerator(EnumPlugIns enum) {
        vars.put(PLUGENUMERATOR, enum);
    }

    public synchronized String getPlugEnumerator() {
        if (checkKey(PLUGENUMERATOR)) {
            return (String)vars.get(PLUGENUMERATOR);
        }
        return "";
    }

    public synchronized void setSplitView(boolean status) {
        setSplitView(booleanToString(status));
    }

    public synchronized void setSplitView(String status) {
        vars.put(SPLITVIEW, status);
    }

    public synchronized boolean getSplitView() {
        return getJMboolean(SPLITVIEW);
    }

    public synchronized void setReleasePause(boolean status) {
        setReleasePause(booleanToString(status));
    }

    public synchronized void setReleasePause(String status) {
        vars.put(RELEASEPAUSE, status);
    }

    public synchronized boolean getReleasePause() {
        return getJMboolean(RELEASEPAUSE);
    }
    /** This returns the base path to our language resource bundles */
    public synchronized String getBundleBase() {
        return "anecho.JamochaMUD.JamochaMUDBundle";
    }

    public synchronized void setMainWindowIconified(boolean status) {
        setMainWindowIconified(booleanToString(status));
    }

    public synchronized void setMainWindowIconified(String iconified) {
        vars.put(MAINWINDOWICONIFIED, iconified);
    }

    public synchronized boolean isMainWindowIconified() {
        return getJMboolean(MAINWINDOWICONIFIED);
    }

    public synchronized void setLocalEcho(boolean status) {
        setLocalEcho(booleanToString(status));
    }

    public synchronized void setLocalEcho(String state) {
        vars.put(LOCALECHO, state);
    }

    public synchronized boolean isLocalEchoEnabled() {
        return getJMboolean(LOCALECHO);
    }

    public synchronized void setOSName(String os) {
        vars.put(OSNAME, os);
    }

    public synchronized String getOSName() {
        return getJMString(OSNAME);
    }

    public synchronized void setAltFocus(String status) {
        vars.put(ALTFOCUS, status);
    }

    public synchronized void setAltFocus(boolean status) {
        setAltFocus(booleanToString(status));
    }

    public synchronized boolean getAltFocus() {
        return getJMboolean(ALTFOCUS);
    }

    public synchronized void setSingleUserMode(boolean status) {
        setSingleUserMode(booleanToString(status));
    }

    public synchronized void setSingleUserMode(String status) {
        vars.put(SINGLEUSERMODE, status);
    }

    public synchronized boolean isSingleUserMode() {
        return getJMboolean(SINGLEUSERMODE);
    }

    /** Add a new user-variable or change the definition of an existing one */
    public synchronized void addVariable(String name, String value) {
        if (userVariables.containsKey(name)) {
            userVariables.remove(name);
        }

        userVariables.put(name, value);

    }

    /** Get the value of the given user-variable name */
    public synchronized String getVariable(String name) {
        if (userVariables.containsKey(name)) {
            return (String)userVariables.get(name);
        }

        return null;

    }

    /** Remove the selected variable from the list of user-variables */
    public synchronized boolean removeVariable(String name) {
        if (userVariables.containsKey(name)) {
            userVariables.remove(name);
            return true;
        }

        return false;
    }

    public synchronized Hashtable getAllVariables() {
        return userVariables;
    }

    public synchronized void addDefinition(String name, String value) {
        if (userDefs.containsKey(name)) {
            userDefs.remove(name);
        }

        userDefs.put(name, value);
    }

    public synchronized String getDefinition(String name) {
        if (userDefs.containsKey(name)) {
            return (String)userDefs.get(name);
        }

        return null;
    }

    public synchronized boolean removeDefinition(String name) {
        if (userDefs.containsKey(name)) {
            userDefs.remove(name);
            return true;
        }

        return false;

    }

    public synchronized Hashtable getAllDefinitions() {
        return userDefs;
    }

    /** Sets up a default MU* as we've had a request for one
     * but don't seem to have any set!
     */
    private void createNewMuck() {
        System.out.println("Create New Muck.");

        Vector tempName = new Vector(0, 1);
        Vector tempAddy = new Vector(0, 1);
        Vector tempPort = new Vector(0, 1);

        tempName.addElement("FurryMuck");
        tempAddy.addElement("muck.furry.com");
        tempPort.addElement("8888");

        setMuckName(tempName);
        setMuckAddy(tempAddy);
        setMuckPort(tempPort);
    }

}                            

