/* This PlugIn will be responsible for handling user
 * defined triggers and gags, configurable 
 * $Id: Trigger.java 1.19 2002/10/24 02:16:08 jeffnik Exp jeffnik $
 */

/* The trigger plug-in and JamochaMUD are
 * Copyright (C) 1998-2002 Jeff Robinson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package anecho.JamochaMUD.plugins;

// These import statements are specific to this plug-in
import anecho.JamochaMUD.JMConfig;
import anecho.JamochaMUD.MuSocket;

// import java.applet.Applet;
// import java.applet.AudioClip;
import sun.audio.*;

import java.awt.*;
import java.awt.event.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.RandomAccessFile;

import java.util.Vector;

import anecho.gui.ResReader;
import anecho.gui.SyncFrame;

/* This PlugIn will be responsible for handling user
 * defined triggers and gags, configurable 
 * @version $Id: Trigger.java 1.19 2002/10/24 02:16:08 jeffnik Exp jeffnik $
 * @author Jeff Robinson
 */

public class Trigger implements PlugInterface, ActionListener, ItemListener {

    // Variables
    SyncFrame ruleFrame;
    boolean inProcess = false;	// this stops multiple instances of the dialogue box
    Choice ruleName = new Choice();
    Vector mainRules = new Vector(0, 1);

    // Variables for rule editing dialogue
    private TextField ruleSetName = new TextField();
    private String theRules;
    private List ruleList = new List();
    private Button tempButton;
    private Button colourButton = new Button();
    private Button mediaButton = new Button();
    private Label tempLabel;
    private JMConfig settings;
    private Choice ruleOption = new Choice();
    private TextField ruleWord = new TextField();
    private Choice colourSample = new Choice();
    private Checkbox colourCheck = new Checkbox();
    private Checkbox launchCheck = new Checkbox();
    private Checkbox mediaCheck = new Checkbox();
    private Checkbox gagCheck = new Checkbox();
    private TextField launchPath = new TextField();
    private TextField customColour = new TextField();
    private TextField mediaPath = new TextField();

    // File locations and paths
    String pathSeparator = new String(java.io.File.separator);
        String plugIns;       // our plugins directory
        String triggerDir;    // the directory where we keep our trigger rules
        File triggerFile;     // the file where we keep our trigger rules


	private static final String RULES = "$RULES:";
	private static final String RULENAME = "$NAME:";
        private static final String COLOUR = "$COLOUR:";
        private static final String LAUNCH = "$LAUNCH:";
        private static final String MEDIA = "$MEDIA:";

        private static final int NO_MATCH = 0;
        private static final int TRUE_MATCH = 1;
        private static final int FALSE_MATCH = 2;

        public void setSettings(JMConfig mainSettings) {
            this.settings = mainSettings;
            plugIns = new String(settings.getUserPlugInDir());
            triggerDir = new String(plugIns + pathSeparator + "TriggerDir");
            triggerFile = new File(triggerDir + pathSeparator + ".trigger.rc");
	}

        /** The human-readable name of our plugin */
	public String PlugInName() {
		return "Gags & Triggers";
	}

        /** A human readable description of our plugin */
	public String PlugInDescription() {
            // return "A plug-in for user-defined gags (hiding unwanted output) and triggers (highlighting input in a number of ways, including sound or colours).";
            ResReader rb = new ResReader("anecho.JamochaMUD.plugins.TriggerDir.TriggerBundle");
            return rb.langString("description");
        }

        /** The type of plug-in that this will be.
         * The current types supported are Output and Input */
	public String PlugInType() {
		return "Output";
	}

	/** The main method of Trigger.  Here we are passed a string from 
	 * main program, will parse it for triggers, and then return whatever
	 * has been dictated the the plugins, or lack thereof
	 */
        public String PlugMain(String jamochaString, MuSocket mu) {
            String finalString = new String(ParseGags(jamochaString));
            return finalString;
	}

	public void PlugInProperties() {
		// We'll call the configuration dialogue for this plugin.
		// Since it is rather extensive, we've put it into its own
		// package.  (That way we can store the user configs there, too).
		if (!inProcess) {
			inProcess = true;
			SelectRule();
		}
	}

	public void InitialiseAtLoad(){
            // We'll call a class in the TriggerDir to scoop any existing .trigger.rc info
            // The triggers must be read at initialisation so that they are available as
            // soon as the MU* client connects.
            ReadTriggers();
	}

	public void SetAtHalt(){
	}

        /** We require a directory for our settings */
        public boolean haveConfig() {
            return true;
        }

        public boolean HasProperties() {
            return true;
        }

	/** Do an "installation check", and read in triggers	
	 * if they already exist.  If not, create the proper space
	 */
	private void ReadTriggers() {

            if (!triggerFile.exists()) return;	// No trigger file, no need to continue

            // We'll open the .trigger.rc file and read in the rulesets
            RandomAccessFile reader;
            boolean loop = true;

            try {
                reader = new RandomAccessFile(triggerFile.toString(), "r");

                String line;
                StringBuffer fullLine = new StringBuffer("");
                while (loop) {
                    try {
                        line = new String(reader.readLine());
                    } catch (Exception e) {
                        // We're all out of lines
                        break;
                    }

                    if (line == null || line.trim().equals("")) {
                        loop = false;
                        break;
                    }

                    line.trim();
                    fullLine.append(line);

                    if (line != null && line.indexOf("$DONE") > 0) {
                        mainRules.addElement(fullLine.toString());
                        fullLine = new StringBuffer("");
                    }

                }

                reader.close();

            } catch(Exception e) {
                System.out.println("Trigger plugin could not access " + triggerFile + ", exception " + e);
                e.printStackTrace();
                return;
            }

            SetChoices();
        }

        /** This displays a list of available rules, allowing the
         * user to add, edit, or delete rules. */
        private void SelectRule() {
            // Fix this XXX!
            // We should set it up so we can querry JamochaMUD as to the location of
            // language resource files
            ResReader rb = new ResReader("anecho.JamochaMUD.plugins.TriggerDir.TriggerBundle");

            ruleFrame = new SyncFrame(rb.langString("title"));
            GridBagLayout layout = new GridBagLayout();
            GridBagConstraints constraints = new GridBagConstraints();
            ruleFrame.setLayout(layout);

            tempLabel = new Label("Rule name:");
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 0;
            constraints.gridy = 0;
            constraints.insets = new Insets(7, 7, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(tempLabel, constraints);
            ruleFrame.add(tempLabel);

            constraints.gridwidth = 3;
            constraints.gridheight = 1;
            constraints.gridx = 1;
            constraints.gridy = 0;
            constraints.insets = new Insets(7, 2, 2, 7);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(ruleName, constraints);
            ruleName.addItemListener(this);
            ruleFrame.add(ruleName);

            tempButton = new Button(rb.langString("addARule"));
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 0;
            constraints.gridy = 1;
            constraints.insets = new Insets(2, 2, 20, 2);
            constraints.fill = GridBagConstraints.BOTH;
            tempButton.setActionCommand("AddNewRule");
            tempButton.addActionListener(this);
            layout.setConstraints(tempButton, constraints);
            ruleFrame.add(tempButton);

            tempButton = new Button(rb.langString("removeThisRule"));
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 1;
            constraints.gridy = 1;
            constraints.insets = new Insets(2, 2, 20, 7);
            constraints.fill = GridBagConstraints.BOTH;
            tempButton.setActionCommand("RemoveThisRule");
            tempButton.addActionListener(this);
            layout.setConstraints(tempButton, constraints);
            ruleFrame.add(tempButton);

            tempButton = new Button(rb.langString("saveAllRules"));
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 2;
            constraints.gridy = 1;
            constraints.insets = new Insets(2, 2, 20, 7);
            constraints.fill = GridBagConstraints.BOTH;
            tempButton.setActionCommand("SaveAllRules");
            tempButton.addActionListener(this);
            layout.setConstraints(tempButton, constraints);
            ruleFrame.add(tempButton);

            tempButton = new Button(rb.langString("cancelAllChanges"));
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 3;
            constraints.gridy = 1;
            constraints.insets = new Insets(2, 2, 20, 7);
            constraints.fill = GridBagConstraints.BOTH;
            tempButton.setActionCommand("CancelAllChanges");
            tempButton.addActionListener(this);
            layout.setConstraints(tempButton, constraints);
            ruleFrame.add(tempButton);

            tempLabel = new Label(rb.langString("name") + ":");
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 0;
            constraints.gridy = 2;
            constraints.insets = new Insets(7, 7, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(tempLabel, constraints);
            ruleFrame.add(tempLabel);

            constraints.gridwidth = GridBagConstraints.REMAINDER;
            constraints.gridheight = 1;
            constraints.gridx = 1;
            constraints.gridy = 2;
            constraints.insets = new Insets(7, 2, 2, 7);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(ruleSetName, constraints);
            ruleFrame.add(ruleSetName);

            ruleList = new List(); // This way we know it is already empty
            ruleList.addItemListener(this);
            constraints.gridwidth = 4;
            constraints.gridheight = 7;
            constraints.weightx = 4;
            constraints.weighty = 7;
            constraints.gridx = 0;
            constraints.gridy = 3;
            constraints.insets = new Insets(2, 7, 2, 7);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(ruleList, constraints);
            ruleFrame.add(ruleList);

            tempButton = new Button(rb.langString("moveRuleUp"));
            tempButton.setActionCommand("MoveRuleUp");
            tempButton.addActionListener(this);
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.weighty = 1;
            constraints.weightx = 1;
            constraints.gridx = 0;
            constraints.gridy = 11;
            constraints.insets = new Insets(2, 7, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(tempButton, constraints);
            ruleFrame.add(tempButton);

            tempButton = new Button(rb.langString("moveRuleDown"));
            tempButton.setActionCommand("MoveRuleDown");
            tempButton.addActionListener(this);
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 1;
            constraints.gridy = 11;
            constraints.insets = new Insets(2, 2, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(tempButton, constraints);
            ruleFrame.add(tempButton);

            tempButton = new Button(rb.langString("newRule"));
            tempButton.setActionCommand("NewRule");
            tempButton.addActionListener(this);
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 2;
            constraints.gridy = 11;
            constraints.insets = new Insets(2, 2, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(tempButton, constraints);
            ruleFrame.add(tempButton);

            tempButton = new Button(rb.langString("deleteRule"));
            tempButton.setActionCommand("DeleteRule");
            tempButton.addActionListener(this);
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 3;
            constraints.gridy = 11;
            constraints.insets = new Insets(2, 2, 2, 7);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(tempButton, constraints);
            ruleFrame.add(tempButton);

            ruleOption = new Choice();
            ruleOption.add("AND");
            ruleOption.add("NOT");
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 0;
            constraints.gridy = 12;
            constraints.insets = new Insets(2, 7, 8, 7);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(ruleOption, constraints);
            ruleFrame.add(ruleOption);

            constraints.gridwidth = 2;
            constraints.gridheight = 1;
            constraints.gridx = 1;
            constraints.gridy = 12;
            constraints.insets = new Insets(2, 2, 8, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(ruleWord, constraints);
            ruleFrame.add(ruleWord);

            tempButton = new Button(rb.langString("updateRule"));
            tempButton.setActionCommand("UpdateRule");
            tempButton.addActionListener(this);
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 3;
            constraints.gridy = 12;
            constraints.insets = new Insets(2, 2, 8, 7);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(tempButton, constraints);
            ruleFrame.add(tempButton);

            gagCheck = new Checkbox(rb.langString("gag"), false);
            gagCheck.addItemListener(this);
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 0;
            constraints.gridy = 13;
            constraints.insets = new Insets(2, 7, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(gagCheck, constraints);
            ruleFrame.add(gagCheck);

            colourCheck = new Checkbox(rb.langString("colour"), false);
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 1;
            constraints.gridy = 13;
            constraints.insets = new Insets(2, 2, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(colourCheck, constraints);
            colourCheck.addItemListener(this);
            ruleFrame.add(colourCheck);

            colourSample = new Choice();
            colourSample.add("custom");
            colourSample.add("black");
            colourSample.add("blue");
            colourSample.add("cyan");
            colourSample.add("dark gray");
            colourSample.add("gray");
            colourSample.add("green");
            colourSample.add("light gray");
            colourSample.add("magenta");
            colourSample.add("orange");
            colourSample.add("pink");
            colourSample.add("red");
            colourSample.add("white");
            colourSample.add("yellow");
            constraints.gridwidth = 2;
            constraints.gridheight = 1;
            constraints.gridx = 2;
            constraints.gridy = 13;
            constraints.insets = new Insets(2, 2, 2, 7);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(colourSample, constraints);
            ruleFrame.add(colourSample);
            colourSample.addItemListener(this);

            // Change the launch checkbox into customisable options
            launchCheck = new Checkbox(rb.langString("selectedOnly"), false);
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 0;
            constraints.gridy = 14;
            constraints.insets = new Insets(2, 7, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(launchCheck, constraints);
            ruleFrame.add(launchCheck);
            launchCheck.setEnabled(false);

            Label tempLabel = new Label(rb.langString("customColour") + ":");
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 1;
            constraints.gridy = 14;
            constraints.insets = new Insets(2, 2, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(tempLabel, constraints);
            ruleFrame.add(tempLabel);

            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 2;
            constraints.gridy = 14;
            constraints.insets = new Insets(2, 2, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(customColour, constraints);
            ruleFrame.add(customColour);
            customColour.setEnabled(false);

            colourButton = new Button(rb.langString("chooseColour"));
            colourButton.setActionCommand("ChooseColour");
            colourButton.addActionListener(this);
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 3;
            constraints.gridy = 14;
            constraints.insets = new Insets(2, 2, 2, 7);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(colourButton, constraints);
            ruleFrame.add(colourButton);
            colourButton.setEnabled(false);

            mediaCheck = new Checkbox(rb.langString("media"), false);
            mediaCheck.addItemListener(this);
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 0;
            constraints.gridy = 15;
            constraints.insets = new Insets(2, 7, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(mediaCheck, constraints);
            ruleFrame.add(mediaCheck);
            mediaCheck.setEnabled(true);

            constraints.gridwidth = 2;
            constraints.gridheight = 1;
            constraints.gridx = 1;
            constraints.gridy = 15;
            constraints.insets = new Insets(2, 2, 2, 2);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(mediaPath, constraints);
            ruleFrame.add(mediaPath);
            mediaPath.setEnabled(false);

            mediaButton = new Button(rb.langString("browse"));
            mediaButton.setActionCommand("BrowseMediaPath");
            mediaButton.addActionListener(this);
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 3;
            constraints.gridy = 15;
            constraints.insets = new Insets(2, 2, 2, 7);
            constraints.fill = GridBagConstraints.BOTH;
            layout.setConstraints(mediaButton, constraints);
            ruleFrame.add(mediaButton);
            mediaButton.setEnabled(false);

            /* "Global" Accept/Cancel for our rules */
            tempButton = new Button(rb.langString("acceptChanges"));
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 0;
            constraints.gridy = 17; // Fix this XXX
            constraints.insets = new Insets(2, 7, 7, 2);
            constraints.fill = GridBagConstraints.BOTH;
            tempButton.setActionCommand("AcceptChanges");
            tempButton.addActionListener(this);
            layout.setConstraints(tempButton, constraints);
            ruleFrame.add(tempButton);

            tempButton = new Button(rb.langString("cancelChanges"));
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.gridx = 3;
            constraints.gridy = 17; // Fix this XXX
            constraints.insets = new Insets(2, 2, 7, 7);
            constraints.fill = GridBagConstraints.BOTH;
            tempButton.setActionCommand("CancelChanges");
            tempButton.addActionListener(this);
            layout.setConstraints(tempButton, constraints);
            ruleFrame.add(tempButton);

            ruleFrame.pack();

            // Make certain we're showing the proper rule information
            // if (ruleName != null) {
                EditAddRule(ruleName.getSelectedIndex());
                // }

            // Show 'em what we're made of!
            ruleFrame.setVisible(true);
        }

        public void actionPerformed(ActionEvent e) {
            String cmd = new String(e.getActionCommand());

            if (cmd.equals("AcceptChanges")) {
                WriteNewRule();
                return;
            }

            if (cmd.equals("CancelChanges")) {
                // User has opted out of changes
                // This rule now only affects the visible trigger, which we will restore
                EditAddRule(ruleName.getSelectedIndex());

                return;
            }

            if (cmd.equals("RemoveThisRule")) {
                // Remove this rule from the list
                if (ruleName.getItemCount() < 1) return; // No rules to remove

                int rule = ruleName.getSelectedIndex();
                ruleName.remove(rule);
                mainRules.removeElementAt(rule);

                // Make certain our rules show the proper info after removal
                EditAddRule(ruleName.getSelectedIndex());

                return;
            }

            if (cmd.equals("SaveAllRules")) {
                // We'll write out all the rules and clean up our frame
                WriteRulesToFile();
                ruleFrame.setVisible(false);
                inProcess = false;
                ruleFrame.dispose();
            }

            if (cmd.equals("CancelAllChanges")) {
                // This is the end of the line for us!
                ruleFrame.setVisible(false);

                // Remove our "lock"
                inProcess = false;

                ruleFrame.dispose();

            }

            if (cmd.equals("AddNewRule")) {
                addBlankRule();
                // mainRules.addElement("$RULES:1$0:AND:Blank$COLOUR:red$LAUNCH:None$MEDIA:None$NAME:Blank$DONE");
                // ruleName.add("Blank");
                // ruleName.select(mainRules.size() - 1);
                // EditAddRule(mainRules.size() - 1);
                return;
            }

            if (cmd.equals("EditThisRule")) {
                EditAddRule(ruleName.getSelectedIndex());
                return;
            }

            if (cmd.equals("UpdateRule")) {
                // UpdateRule(ruleName.getSelectedIndex());
                UpdateRule();
                return;
            }

            if (cmd.equals("MoveRuleUp")) {
                if (ruleList.getSelectedItem() != null && ruleList.getSelectedIndex() > 0) {
                    MoveRule(ruleList.getSelectedIndex(), -1);
                }

                return;
            }

            if (cmd.equals("MoveRuleDown")) {
                if (ruleList.getSelectedItem() != null & ruleList.getSelectedIndex() < (ruleList.getItemCount() - 1)) {
                    MoveRule(ruleList.getSelectedIndex(), 1);
                }
                return;
            }

            if (cmd.equals("NewRule")) {
                AddNewSubRule();
                return;
            }

            if (cmd.equals("DeleteRule")) {
                if (ruleList.getSelectedItem() != null) {
                    DeleteSubRule(ruleList.getSelectedIndex());
                }
                return;
            }

            if (cmd.equals("BrowseMediaPath")) {
                selectFile(mediaPath, mediaPath.getText());
            }

            if (cmd.equals("ChooseColour")) {
                // Create our colour selector!!
                String oldColour = customColour.getText();

                // Create a valid colour
                if (oldColour.length() < 6 || oldColour.length() > 7) {
                    oldColour = new String("FFFFFF");
                } else {
                    oldColour = oldColour.substring(1, 7);
                }

                int red, green, blue, fullColour;
                try {
                    red = Integer.parseInt(oldColour.substring(0, 2), 16);
                    green = Integer.parseInt(oldColour.substring(2, 4), 16);
                    blue = Integer.parseInt(oldColour.substring(4, 6), 16);
                } catch (Exception colourErr) {
                    // Not a valid number
                    red = 255;
                    green = 255;
                    blue = 255;
                }

                java.awt.Color baseColour = new java.awt.Color(red, green, blue);

                anecho.gui.ColourPicker chooser = new anecho.gui.ColourPicker(ruleFrame, "Choose a colour", baseColour);

                // set the colour selector visible
                chooser.show();

                // update customColour textArea
                if (chooser.getColour() != null) {
                    customColour.setText("#" + chooser.getHexColour());
                }

                return;
            }
        }

        /** Our item state changed.  If it was the checkbox on the
         * &quot;Gag" feature then we toggle the rest of the
         * trigger actions on or off, accordinly.
         */
        public void itemStateChanged(ItemEvent event) {
            if (event.getItem() == gagCheck.getLabel()) {
                if (gagCheck.getState()) {
                    // All other options are disabled as the "GAG" has been selected
                    SetRuleOptions(false);
                    // setMediaState(false);
                } else {
                    // Make all options available as the "GAG" has been deselected
                    SetRuleOptions(true);
                    // setMediaState(true);
                }
                return;
            }

            if (event.getItem() == mediaCheck.getLabel()) {
                // mediaPath.setEnabled(mediaCheck.getState());
                // mediaButton.setEnabled(mediaCheck.getState());
                setMediaState(mediaCheck.getState());
                return;
            }

            if (event.getSource() == colourCheck) {
                // Selectively enable/disable colour-associated items
                // colourSample.setEnabled(false);
                toggleCustomColour(colourCheck.getState());
                return;
            }

            if (event.getSource() == colourSample) {
                // If our colourSample has changed to &quot;custom" then
                // set the rest of our colour widgets accordingly.
                toggleCustomColour(true);
                return;
            }

            if (event.getSource() == ruleName) {
                // Our rule has changed, show the new rule information
                EditAddRule(ruleName.getSelectedIndex());
                return;
            }

            if (event.getSource() == ruleList) {
                // We've selected a new item, lets put its values
                // in the appropriate areas
                int selected = ruleList.getSelectedIndex();
                showRuleWord(selected);
                return;
            }

        }

        /** read the new vector into the choices
         * for our main dialogue
         */
        private void SetChoices() {
            ruleName.removeAll();
            String tempName;
            int mRuleSize = mainRules.size();

            // for (int i = 0; i < mainRules.size(); i++) {
            for (int i = 0; i < mRuleSize; i++) {
                tempName = new String(PullRule(mainRules.elementAt(i), RULENAME));

                if (tempName != null) {
                    ruleName.add(tempName);
                }
            }

            // Show information about our rule
            EditAddRule(ruleName.getSelectedIndex());
        }

        /** Go through the rules and see if have anything that matches */
        private String ParseGags(String input) {

            if (mainRules == null || mainRules.size() < 1) {
                // We have no rules yet
                return input;
            }

            StringBuffer grinder = new StringBuffer(input);

            // Loop through all the rules...
            int numRules = 0;
            boolean gag;
            int ruleResult = NO_MATCH;

            int mRuleSize = mainRules.size();

            // for (int set = 0; set < mainRules.size(); set++) {
            for (int set = 0; set < mRuleSize; set++) {

                gag = false; // Reset this for each rule, otherwise we can have "fall through"

                // How many rules are in this rule set?
                numRules = Integer.parseInt(PullRule(mainRules.elementAt(set), RULES));

                for (int j = 0; j < numRules; j++) {
                    ruleResult = CheckRule(input, set, j);

                    if (ruleResult == TRUE_MATCH) {
                        gag = true;
                    }
                    if (ruleResult == FALSE_MATCH) {
                        gag = false;
                        break;
                    }
                }

                // We'll see if this gag is appropriate...
                if (gag) {
                    grinder = new StringBuffer(ApplyGag(grinder, set));

                    // If audio is applicable, we'll call that as well!
                    String rule = PullRule(mainRules.elementAt(set), MEDIA);
                    if (!rule.equals("None")) {
                        playMedia(PullRule(mainRules.elementAt(set), MEDIA));
                    }

                }

            }

            return grinder.toString();
        }

        /** Check the passed string to see if it matches one of our rules
         * @param input The input string that the rule will be checked against
         * @param set The rule-set to check.  A rule-set is a complete rule.
         * @param rule The specific rule within a set to check against.  Usually a condition.
         */
        private int CheckRule(String input, int set, int rule) {
            String chkRule = new String(PullRule(mainRules.elementAt(set), "$" + rule + ":"));
            String match = (chkRule.substring(chkRule.indexOf(":") + 1)).toLowerCase();
            input = input.toLowerCase();
            int state = NO_MATCH;

            if (chkRule.startsWith("AND")) {
                if (input.indexOf(match) > 0 || input.startsWith(match)) {
                    state = TRUE_MATCH;
                } else {
                    // We didn't meet an "AND" condition, so this trigger fails
                    state = FALSE_MATCH;
                }
            }

            if (chkRule.startsWith("OR")) {
                // if (input.indexOf(match) > 0 || input.startsWith(match)) {
                if (input.indexOf(match) > 0 || input.startsWith(match)) {
                    state = TRUE_MATCH;
                }
            }

            if (chkRule.startsWith("NOT")) {
                if (input.indexOf(match) > 0 || input.startsWith(match)) {
                    state = FALSE_MATCH;
                }
            }

            return state;
        }

        /** Apply the Gag to the String we were passed from the main program */
        private String ApplyGag(StringBuffer sb, int set) {
            String colour = PullRule(mainRules.elementAt(set), COLOUR);
            colour = new String(colour.toLowerCase());
            String colourCode = new String();
            if (colour.equals("gag")) {
                return "";	// Not a colour, but a gag!
            }

            // Make the colour our colour name by default, change if necessary

            if (colour.startsWith("#")) {
                colourCode = '\u001b' + "[" + colour + "m";
            }

            if (colour.equals("black")) colourCode = '\u001b' + "[30m";
            if (colour.equals("red")) colourCode = '\u001b' + "[31m";
            if (colour.equals("green")) colourCode = '\u001b' + "[32m";
            if (colour.equals("yellow")) colourCode = '\u001b' + "[33m";
            if (colour.equals("blue")) colourCode = '\u001b' + "[34m";
            if (colour.equals("magenta")) colourCode = '\u001b' + "[35m";
            if (colour.equals("cyan")) colourCode = '\u001b' + "[36m";
            if (colour.equals("white")) colourCode = '\u001b' + "[37m";

            if (sb.length() > 0 && sb.charAt(0) != '\u001b') {
                sb.insert(0, colourCode);
                sb.append('\u001b' + "[0m");
            }

            return sb.toString();
        }

        /** Search through the string for the rule name,
         * and extract its information
         */
        private String PullRule(Object ruleObj, String ruleName) {
            String rule = ruleObj.toString();
            int start, end;
            start = rule.indexOf(ruleName);
            start = start + ruleName.length();
            end = rule.indexOf("$", start + 1);

            return rule.substring(start, end);
        }

        /** Modify a given rule.*/
        private void ChangeRule(int item, String option, String ruleWord) {
            int rules = Integer.parseInt(PullRule(theRules, RULES));
            StringBuffer workString = new StringBuffer("$RULES:" + rules);

            for (int i = 0; i < rules; i++) {
                if (i == item) {
                    // This is the rule we change
                    workString.append("$" + i + ":" + option + ":" + ruleWord);
                } else {
                    workString.append("$" + i + ":" + (PullRule(theRules, "$" + i + ":")));
                }
            }

            workString.append("$COLOUR:" + (PullRule(theRules, COLOUR)));
            workString.append("$LAUNCH:" + (PullRule(theRules, LAUNCH)));
            workString.append("$MEDIA:" + (PullRule(theRules, MEDIA)));
            workString.append("$NAME:" + (PullRule(theRules, RULENAME)));
            workString.append("$DONE");

            // Copy this as our new working rule, and then update the display
            theRules = new String(workString.toString());
            // RefreshRules();
        }

        /** Add a new rule or edit an existing rule
         * -1 indicates that we'll be dealing with a
         * new rule, otherwise we'll edit the rule
         * number passed by the int.
         */
        private void EditAddRule(int rule) {
            // There is a posibility that we don't have any rules yet
            if (mainRules == null || mainRules.size() == 0) {
                // Create a new blank rule and set our pointer to that
                addBlankRule();
                rule = 0;
            }

            theRules = new String((String)mainRules.elementAt(rule));

            if (rule >= 0) {
                ruleSetName.setText(PullRule(theRules, RULENAME));
                SetupRules(); // Fill in the appropriate rule areas
            }
        }

        /** This will read in a rule, breaking it up
         * for nice display in our little rule-mangler!
         */
        private void SetupRules() {
            // First, we'll check to see if this is a GAG,
            // and if so, we'll disable half our options now
            if (PullRule(theRules, COLOUR).equalsIgnoreCase("GAG")) {
                gagCheck.setState(true);
                SetRuleOptions(false);
            } else {
                // Better safe than sorry, no?
                gagCheck.setState(false);
                SetRuleOptions(true);
                // Set colour information
                String cName = PullRule(theRules, COLOUR).toLowerCase();
                if (cName.startsWith("#")) {
                    // This is a custom colour
                    customColour.setEnabled(true);
                    customColour.setText(cName);
                    colourSample.select("custom");
                    colourButton.setEnabled(true);
                } else {
                    customColour.setText("");
                    customColour.setEnabled(false);
                    colourSample.select(cName);
                    colourButton.setEnabled(false);
                }
                colourCheck.setState(true);
            }

            // Fill in the media information
            String text = PullRule(theRules, MEDIA);
            if (text.equals("None")) {
                mediaPath.setText("");
                mediaCheck.setState(false);
                setMediaState(false);
            } else {
                mediaPath.setText(text);
                mediaCheck.setState(true);
                setMediaState(true);
            }

            RefreshRules();

            // For the convenience of the user, select the first rule
            ruleList.select(0);
            showRuleWord(0);

        }

        /** Set up our ruleWord textArea and associated "choice"
         */
        private void showRuleWord(int selected) {
            String ruleType, tag;
            ruleType = new String(PullRule(theRules, "$" + selected + ":"));
            tag = (ruleType.substring(ruleType.indexOf(":") + 1));

            ruleOption.select(ruleType.substring(0, ruleType.indexOf(":")));
            ruleWord.setText(tag);
        }

        /** This enables or disables options available,
         * depending on the boolean passed
         */
        private void SetRuleOptions(boolean stat) {
            colourCheck.setEnabled(stat);

            mediaCheck.setEnabled(stat);
            setMediaState(mediaCheck.getState());
        }

        /** Parse the set of rules, writing it out
         * to our list
         */
        private void RefreshRules() {
            // Get the number of rules that we're dealing with
            String ruleType, tag;
            int start;
            int num = Integer.parseInt(PullRule(theRules, RULES));
            if (num < 1) return;// This shouldn't happen... but y'never know

            ruleList.removeAll();
            for (int i = 0; i < num; i++) {
                ruleType = new String(PullRule(theRules, "$" + i + ":"));
                tag = (ruleType.substring(ruleType.indexOf(":") + 1));
                ruleList.add(ruleType.substring(0, ruleType.indexOf(":")) + " -> " + tag);
            }
        }

        /** Modify an existing rule */
        private void UpdateRule() {
            if (ruleList.getSelectedItem() == null || ruleWord.getText().equals("")) return;

            int item = ruleList.getSelectedIndex();
            ChangeRule(item, ruleOption.getSelectedItem(), ruleWord.getText());
            RefreshRules();
        }

        private void MoveRule(int item, int direction) {
            int size = ruleList.getItemCount();
            Vector tempItem = new Vector(0, 1);
            String itemString;

            for (int i = 0; i < size; i++) {
                tempItem.addElement(PullRule(theRules, "$" + i + ":"));
            }

            itemString = new String((String)tempItem.elementAt(item));
            tempItem.removeElementAt(item);
            tempItem.insertElementAt(itemString, item + direction);

            int colon;
            for (int i = 0; i < size; i++) {
                itemString = new String((String)tempItem.elementAt(i));
                colon = itemString.indexOf(":");
                ChangeRule(i, itemString.substring(0, colon), itemString.substring(colon + 1));
            }
            RefreshRules();
            ruleList.select(item + direction);
        }

        /** Write out our new rule in the proper format and
         * then update the main window */
        private void WriteNewRule() {
            int rules = Integer.parseInt(PullRule(theRules, RULES));
            StringBuffer workString = new StringBuffer("$RULES:" + rules);

            for (int i = 0; i < rules; i++) {
                workString.append("$" + i + ":" + (PullRule(theRules, "$" + i + ":")));
            }

            // Now, glean the rest of the info from out dialogues
            workString.append("$COLOUR:");
            if (gagCheck.getState()) {
                workString.append("GAG");
            } else {
                // Check whether we have a custom colour or not
                if (colourSample.getSelectedItem().equals("custom")) {
                    String cusColour = customColour.getText();
                    if (cusColour.length() >= 6 && cusColour.length() <= 7) {
                        if (cusColour.length() == 6) {
                            workString.append("#" + customColour.getText());
                        } else {
                            workString.append(customColour.getText());
                        }
                    } else {
                        workString.append("#ffffff");
                    }
                } else {
                    workString.append(colourSample.getSelectedItem());
                }
            }
            workString.append("$LAUNCH:");
            workString.append("None");

            workString.append("$MEDIA:");
            if (mediaCheck.getState() && !mediaPath.getText().equals("")) {
                workString.append(mediaPath.getText());
            } else {
                workString.append("None");
            }
            workString.append("$NAME:");
            workString.append(ruleSetName.getText());
            workString.append("$DONE");

            // Finished building the new rule, now we'll put it back in
            // into the "main array"
            mainRules.setElementAt(workString.toString(), ruleName.getSelectedIndex());
            SetChoices();
            ruleName.select(PullRule(workString.toString(), RULENAME));
            EditAddRule(ruleName.getSelectedIndex());
        }

        /** Upon approval by the user, we'll write the rules back to the file */
        private void WriteRulesToFile() {

            if (triggerFile.exists()) {
                triggerFile.delete();
            }

            String writeOut;
            // We'll open the .trigger.rc file and read in the rulesets
            RandomAccessFile writer;

            try {
                writer = new RandomAccessFile(triggerFile.toString(), "rw");

                int mRuleSize = mainRules.size();

                // for (int i = 0; i < mainRules.size(); i++) {
                for (int i = 0; i < mRuleSize; i++) {
                    writeOut = new String((String)mainRules.elementAt(i));
                    writer.writeBytes(writeOut.trim() + '\n');
                }

                writer.close();

            } catch (Exception e) {
                System.out.println("Some sort've error while writing out plugins" + e);
            }

        }

        /** Add an additional subrule to an existing rule */
        private void AddNewSubRule() {
            int rules = Integer.parseInt(PullRule(theRules, RULES));
            StringBuffer workString = new StringBuffer("$RULES:" + (rules + 1));

            for (int i = 0; i < rules; i++) {
                workString.append("$" + i + ":" + (PullRule(theRules, "$" + i + ":")));
            }

            workString.append("$" + (rules) + ":" + "AND:blank");

            // Now, glean the rest of the info from out dialogues
            workString.append("$COLOUR:");
            workString.append(PullRule(theRules, COLOUR));
            workString.append("$LAUNCH:");
            workString.append(PullRule(theRules, LAUNCH));
            workString.append("$MEDIA:");
            workString.append(PullRule(theRules, MEDIA));
            workString.append("$NAME:");
            workString.append(PullRule(theRules, RULENAME));
            workString.append("$DONE");

            theRules = new String(workString.toString());
            RefreshRules();
            ruleList.select(rules);
            showRuleWord(rules);
        }

        /** Delete a subrule from our current set of working rules. */
        private void DeleteSubRule(int ruleIndex) {
            int count = 0;
            int rules = Integer.parseInt(PullRule(theRules, RULES));
            StringBuffer workString = new StringBuffer("$RULES:" + (rules - 1));

            for (int i = 0; i < rules; i++) {
                if (i != ruleIndex) {
                    workString.append("$" + count + ":" + (PullRule(theRules, "$" + i + ":")));
                    count++;
                }
            }

            // Now, glean the rest of the info from out dialogues
            workString.append("$COLOUR:");
            workString.append(PullRule(theRules, COLOUR));
            workString.append("$LAUNCH:");
            workString.append(PullRule(theRules, LAUNCH));
            workString.append("$MEDIA:");
            workString.append(PullRule(theRules, MEDIA));
            workString.append("$NAME:");
            workString.append(PullRule(theRules, RULENAME));
            workString.append("$DONE");

            // Finished building the new rule, now we'll put it back in
            // into the "main array"
            theRules = new String(workString.toString());
            RefreshRules();
        }

        /** Show a file-selection dialogue and the insert the results
         * from the selection into the proper location
         */
        private void selectFile(TextField text, String oldPath) {

            FileDialog fd = new FileDialog(ruleFrame, "Select a file", FileDialog.LOAD);

            if (!oldPath.equals("")) {
                fd.setFile(oldPath);
            }

            fd.setVisible(true);

            if (fd.getFile() != null && fd.getFile() != "") {
                text.setText(fd.getDirectory() + fd.getFile());
            }

            return;
        }

        /** Toggle the state of the "media"-related widgets.  This
         * would generally be a simple matter but for the fact that
         * they are also controlled by the "GAG" checkbox
         */
        private void setMediaState(boolean stat) {
            if (gagCheck.getState()) {
                mediaCheck.setEnabled(false);
                mediaPath.setEnabled(false);
                mediaButton.setEnabled(false);
            } else {
                mediaCheck.setEnabled(true);
                mediaPath.setEnabled(stat);
                mediaButton.setEnabled(stat);
            }
        }

        /** Play media is triggered, so let's make some noise! */
        private void playMedia(String audioFileName) {
            // This stuff is drawn from the Applet.AudioClip class.  Yuckyweird.
            // try {
            // AudioClip myClip = getAudioClip(new URL(getCodeBase(), "clip.au"));
            // } catch (Exception e) {
            // }
            try {
                AudioDataStream audioDataStream;
                AudioPlayer audioPlayer = AudioPlayer.player;
                FileInputStream fis = new FileInputStream( new File(audioFileName) );
                AudioStream as = new AudioStream( fis ); // header plus audio data
                AudioData ad = as.getData(); // audio data only, no header
                audioDataStream = new AudioDataStream( ad );
                audioPlayer.start( audioDataStream );
            } catch (Exception e) {
                System.out.println("An exception occured trying to play this file:");
                System.out.println(e);
            }

        }

        private void toggleCustomColour(boolean state) {
            if (colourSample.getSelectedItem().equals("custom")) {
                customColour.setEnabled(state);
                colourButton.setEnabled(state);
            } else {
                // No matter what our state, if colourSample is not custom
                // these options must be disabled
                customColour.setEnabled(false);
                colourButton.setEnabled(false);
            }
        }

        private void addBlankRule() {
            mainRules.addElement("$RULES:1$0:AND:Blank$COLOUR:red$LAUNCH:None$MEDIA:None$NAME:Blank$DONE");
            ruleName.add("Blank");
            ruleName.select(mainRules.size() - 1);
            EditAddRule(mainRules.size() - 1);
        }

}
